package	jp.co.sds_corp.transfer;

import java.io.Serializable;
import java.util.Vector;

/**
* データ受信通知を管理するProxyです。
*/
public class ReceiverSupport implements Serializable
{
	private static final long serialVersionUID = 1L;
	transient Vector<IReceiveListener>	vecAllLsnr = new Vector<IReceiveListener>();
	transient Vector<IReceiveListener>	vecLsnr = new Vector<IReceiveListener>();
	Vector<IDataKey>	vecKey = new Vector<IDataKey>();

	/**
	* debug文を出力します。
	*/
	public void printDebug() {
		for ( int i = 0; i < vecAllLsnr.size(); i++ ) {
			IReceiveListener l = (IReceiveListener )vecAllLsnr.get( i );
			System.out.println( "ALL:" + l );
		}
		for ( int i = 0; i < vecLsnr.size(); i++ ) {
			IDataKey key = (IDataKey )vecKey.get( i );
			IReceiveListener l = (IReceiveListener )vecLsnr.get( i );
			System.out.println( key + ":" + l );
		}
	}

	/**
	* データ受信を受け取るオブジェクトを追加します。
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public synchronized void addReceiveListener( IReceiveListener l ) {
		vecAllLsnr.add( l );
	}

	/**
	* データ受信を受け取るオブジェクトを削除します。
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public synchronized void removeReceiveListener( IReceiveListener l ) {
		vecAllLsnr.remove( l );
	}

	/**
	* データ受信を受け取るオブジェクトを追加します。
	* @param	key		対象のキー
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public synchronized void addReceiveListener( IDataKey key,
								IReceiveListener l )
	{
		vecKey.add( key );
		vecLsnr.add( l );
	}

	/**
	* データ受信を受け取るオブジェクトを削除します。
	* @param	key		対象のキー
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public synchronized void removeReceiveListener( IDataKey key,
								IReceiveListener l )
	{
		vecKey.remove( key );
		vecLsnr.remove( l );
	}

	/**
	* データ受信イベントを登録されているListenerに通知します。
	* @param	e		データ受信イベント
	*/
	public synchronized void fireReceive( ReceiveEvent e )
		throws Exception
	{
		for ( int i = 0; i < vecAllLsnr.size(); i++ ) {
			IReceiveListener l = (IReceiveListener )vecAllLsnr.get( i );
			l.received( e );
		}
		IDataKey key = e.getData().getKey();
		for ( int i = 0; i < vecKey.size(); i++ ) {
			IDataKey k = (IDataKey )vecKey.get( i );
			IReceiveListener l = (IReceiveListener )vecLsnr.get( i );
			if ( k != null ) {
				if ( k.match( key )) {
					l.received( e );
				}
			}
		}
	}

	/**
	* 状態変更イベントを登録されているListenerに通知します。
	* @param	e		状態変更イベント
	*/
	public synchronized void fireStatusChange( StatusChangeEvent e )
		throws Exception
	{
		for ( int i = 0; i < vecAllLsnr.size(); i++ ) {
			IReceiveListener l = (IReceiveListener )vecAllLsnr.get( i );
			l.statusChanged( e );
		}
		for ( int i = 0; i < vecKey.size(); i++ ) {
			IReceiveListener l = (IReceiveListener )vecLsnr.get( i );
			l.statusChanged( e );
		}
	}

}
