package	jp.co.sds_corp.transfer.filter;

import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* データの受信日時によってフィルタリングします。
TimeRange要素によって受信日時を制限することができます。
TimeRange要素の詳細については{@link jp.co.sds_corp.util.daemon.filter.TimeFilter}を参照してください。
なお、TimeRange要素は複数指定することが可能です。
<P>
loglevel要素によって、このフィルタで排除されたデータ受信イベントのログ出力を
制御することができます。
localevel要素には、off,debug,info,warn,error,fatalの各レベルを指定可能です。
<PRE>
&lt;filter class="jp.co.sds_corp.transfer.filter.TimeFilter"&gt;
	&lt;TimeRange type="allow" from="06:00:00" to="21:00:00"/&gt;
	&lt;TimeRange type="deny" from="09:00:00" to="09:05:00"/&gt;
	&lt;TimeRange type="deny"&gt;
		&lt;From&gt;
			&lt;dayofweek&gt;金&lt;/dayofweek&gt;
			&lt;time&gt;18:00:00&lt;/time&gt;
		&lt;/From&gt;
		&lt;To&gt;
			&lt;dayofweek&gt;金&lt;/dayofweek&gt;
			&lt;time&gt;18:00:00&lt;/time&gt;
		&lt;/To&gt;
	&lt;/TimeRange&gt;
	&lt;loglevel&gt;off&lt;/loglevel&gt;
&lt;/filter>
</PRE>
*/
public class TimeFilter extends AbstractFilter
{
	private static final long serialVersionUID = 1L;
	Vector<jp.co.sds_corp.util.daemon.filter.TimeFilter>	vecRange;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		vecRange = new Vector<jp.co.sds_corp.util.daemon.filter.TimeFilter>();
		Iterator<IConfiguration> itr = config.getChilds( "TimeRange" );
		while ( itr.hasNext() ) {
			IConfiguration c = (IConfiguration )itr.next();
			jp.co.sds_corp.util.daemon.filter.TimeFilter range =
				new jp.co.sds_corp.util.daemon.filter.TimeFilter();
			range.init( c );
			vecRange.add( range );
		}
	}

	/**
	* データのフィルタリングを行なう
	* @param	e		データ受信イベント
	*/
	public boolean isValid( ReceiveEvent e ) throws Exception {
		boolean bAllow = false;
		boolean bDeny = false;
		CmnDate now = e.getData().getUpdateTime();
		for ( int i = 0; i < vecRange.size(); i++ ) {
			jp.co.sds_corp.util.daemon.filter.TimeFilter range =
				(jp.co.sds_corp.util.daemon.filter.TimeFilter )vecRange.get( i );
			boolean bIn = range.isValid();
			if ( bIn ) {
				if ( range.getType() ==
					jp.co.sds_corp.util.daemon.filter.TimeFilter.Type.ALLOW )
				{
					bAllow = true;
				} else {
					bDeny = true;
					break;
				}
			}
		}
		if ( bDeny ) {
			return false;
		}
		return bAllow;
	}

	/**
	* データがフィルタリングの対象であるかチェックする。
	* @param	e		データ受信イベント
	*/
	public boolean isTarget( ReceiveEvent e ) throws Exception {
		return true;
	}

	/**
	* このフィルタの文字列表現を返します。
	* @return	このフィルタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += ",TimeRange=[";
		for ( int i = 0; i < vecRange.size(); i++ ) {
			if ( i > 0 ) {
				ret += ",";
			}
			ret += vecRange.get( i );
		}
		ret += "]";
		return ret;
	}
}
